-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_AddAnimPoint"

-- **************************************************
-- General information about this script
-- **************************************************

DS_AddAnimPoint = {}

DS_AddAnimPoint.STR_SHARPCORNERS = DS_AddAnimPoint.BASE_STR

function DS_AddAnimPoint:Name()
	return "Add animated Point"
end

function DS_AddAnimPoint:Version()
	return "0.1"
end

function DS_AddAnimPoint:Description()
	return "Add a point in animation"
end


function DS_AddAnimPoint:Creator()
	return "dauid.com"
end

function DS_AddAnimPoint:UILabel()
	return"Add animated point"
end

function DS_AddAnimPoint:LoadPrefs(prefs)
	self.sharpCorners = prefs:GetBool("DS_AddAnimPoint.sharpCorners", false)
end

function DS_AddAnimPoint:SavePrefs(prefs)
	prefs:SetBool("DS_AddAnimPoint.sharpCorners", self.sharpCorners)
end

function DS_AddAnimPoint:ResetPrefs()
	self.sharpCorners = false
end

function DS_AddAnimPoint:NonDragMouseMove()
	return true -- Call MouseMoved() even if the mouse button is not down
end

-- **************************************************
-- Recurring values
-- **************************************************

DS_AddAnimPoint.numSel = 0
DS_AddAnimPoint.selID = 0
DS_AddAnimPoint.startWeldVec = LM.Vector2:new_local()
DS_AddAnimPoint.startWeldVec:Set(-10000000, -10000000)
DS_AddAnimPoint.startWeldToPoint = true

-- **************************************************
-- The guts of this script
-- **************************************************

function DS_AddAnimPoint:IsEnabled(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function DS_AddAnimPoint:IsRelevant(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	return true
end

function DS_AddAnimPoint:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.startWeldVec:Set(-10000000, -10000000)

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local closestID = -1
	local closest = 1e6
	local curveID = -1
	local segID = -1
	local sharpCorners = self.sharpCorners

	if (mouseEvent.ctrlKey) then
		sharpCorners = not sharpCorners
	end

	mesh:SelectNone()
	

	if (closest < LM_TranslatePoints.autoWeldRadius * LM_TranslatePoints.autoWeldRadius and not(mouseEvent.altKey)) then -- add as an extension to an existing point
		
	else
		local curveID = -1
		local segID = -1

		if (not mouseEvent.altKey) then
			local pickWidth = 3
			while (pickWidth < LM_TranslatePoints.autoWeldRadius) do
				curveID = -1
				segID = -1
				curveID, segID = mouseEvent.view:PickEdge(mouseEvent.pt, curveID, segID, pickWidth)
				if (curveID >= 0) then
					break
				end
				pickWidth = pickWidth + 3
			end
		end
		if (curveID >= 0) then -- add a point in the middle of some curve
			if (moho.gridOn) then
				moho:SnapToGrid(mouseEvent.vec)
			end
			local v = mesh:Curve(curveID):ClosestPointOnSegment(segID, mouseEvent.vec)
			local v2 = LM.Vector2:new_local()
			local percentage = 0
			for i = 0, 100 do
				v2 = mesh:Curve(curveID):PointOnSegment(segID, i*0.01)
				
				if math.floor(v.x*100) == math.floor(v2.x*100) and math.floor(v.y*100) == math.floor(v2.y*100) then
					percentage = i*0.01
					break
				end
			end
			local newpoints = {}
			local startframe = moho.frame
			for i=0, mesh:Curve(curveID):CountPoints() do
				local pt = mesh:Curve(curveID):Point(i)
				local ch = pt.fAnimPos
				for f = 0, ch:Duration() do
					if ch:HasKey(f) then
						moho:SetCurFrame(f)
						newpoints[f] = mesh:Curve(curveID):PointOnSegment(segID, percentage)
						moho:SetCurFrame(startframe)
					end
				end
			end
			
			mesh:AddPoint(newpoints[0], curveID, segID, 0)
			
			--mesh:AddPoint(v2, curveID, segID, moho.layerFrame)
			local lastPoint = mesh:Point(mesh:CountPoints() - 1)
			for k, v in pairs(newpoints) do
				lastPoint.fAnimPos:SetValue(k, v)
			end
			moho:SetCurFrame(0)
			moho:SetCurFrame(startframe)
			
		else -- add a new curve segment
			
		end
		if (sharpCorners) then
			mesh:Point(mesh:CountPoints() - 1):SetCurvature(MOHO.PEAKED, 0)
		end
	end

	--LM_TranslatePoints.selID = mesh:CountPoints() - 1
	--LM_TranslatePoints:OnMouseDown(moho, mouseEvent)

	mouseEvent.view:DrawMe()
end

function DS_AddAnimPoint:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (mesh:CountPoints() < 1) then
		return
	end
	
	if (not LM_TranslatePoints.dragging) then

		if (mouseEvent.altKey) then
			if (self.startWeldVec.x > -1000000) then
				self.startWeldVec:Set(-10000000, -10000000)
				mouseEvent.view:DrawMe()
			end
			return
		end

		-- Display the upcoming weld points for the user's benefit
		local m = LM.Matrix:new_local()
		local testVec1 = LM.Vector2:new_local()
		local testVec2 = LM.Vector2:new_local()
		local p1 = LM.Point:new_local()
		local p2 = LM.Point:new_local()
		local dist = 0
		local curveID = -1
		local segID = -1
		local pickWidth = 3
		local id = mesh:ClosestPoint(mouseEvent.vec)
		local pt

		self.startWeldVec:Set(-10000000, -10000000)
		if (id >= 0) then
			pt = mesh:Point(id)
			testVec1:Set(mouseEvent.vec)
			testVec2:Set(pt.fPos)
			moho.layer:GetFullTransform(moho.frame, m, moho.document)
			m:Transform(testVec1)
			moho.view:Graphics():WorldToScreen(testVec1, p1)
			m:Transform(testVec2)
			moho.view:Graphics():WorldToScreen(testVec2, p2)

			p2.x = p1.x - p2.x
			p2.y = p1.y - p2.y
			dist = (p2.x * p2.x) + (p2.y * p2.y)
			if (dist < LM_TranslatePoints.autoWeldRadius * LM_TranslatePoints.autoWeldRadius) then
				self.startWeldToPoint = true
				self.startWeldVec:Set(pt.fPos)
				mouseEvent.view:DrawMe()
				return
			end
		end

		while (pickWidth < LM_TranslatePoints.autoWeldRadius) do
			curveID, segID = moho.view:PickEdge(p1, curveID, segID, pickWidth)
			if (curveID >= 0) then -- add a point in the middle of some curve
				self.startWeldToPoint = false
				self.startWeldVec:Set(mesh:Curve(curveID):ClosestPointOnSegment(segID, mouseEvent.vec))
				break
			end
			pickWidth = pickWidth + 3
		end
		moho:UpdateUI()
		mouseEvent.view:DrawMe()
		return
	end

	--LM_TranslatePoints:OnMouseMoved(moho, mouseEvent)
end

function DS_AddAnimPoint:OnMouseUp(moho, mouseEvent)
	self.startWeldVec:Set(-10000000, -10000000)
	mouseEvent.ctrlKey = false
	--LM_TranslatePoints:OnMouseUp(moho, mouseEvent)
end

function DS_AddAnimPoint:OnKeyDown(moho, keyEvent)
	--LM_TranslatePoints:OnKeyDown(moho, keyEvent)
end

function DS_AddAnimPoint:DrawMe(moho, view)
	if (LM_TranslatePoints.fillingShape) then
		local g = view:Graphics()
		local matrix = LM.Matrix:new_local()

		moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
		g:Push()
		g:ApplyMatrix(matrix)

		view:DrawPreviewShape()

		g:Pop()
	elseif (not LM_TranslatePoints.dragging) then
		local g = view:Graphics()
		local matrix = LM.Matrix:new_local()
	
		moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
		g:Push()
		g:ApplyMatrix(matrix)

		if (self.startWeldToPoint) then
			g:SetColor(0, 255, 0, 128)
		else
			g:SetColor(255, 0, 0, 128)
		end
		g:SetSmoothing(true)
		g:SetBezierTolerance(2)
		g:FillCircle(self.startWeldVec, moho:PixelToDoc(LM_TranslatePoints.autoWeldRadius) / g:CurrentScale(false))
		g:SetSmoothing(false)
	
		g:Pop()
		return
	end
	LM_TranslatePoints:DrawMe(moho, view)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

DS_AddAnimPoint.DUMMY = MOHO.MSG_BASE
DS_AddAnimPoint.AUTOWELD = MOHO.MSG_BASE + 1
DS_AddAnimPoint.AUTOFILL = MOHO.MSG_BASE + 2
DS_AddAnimPoint.SHARPCORNERS = MOHO.MSG_BASE + 3
DS_AddAnimPoint.SELECTITEM = MOHO.MSG_BASE + 4
DS_AddAnimPoint.sharpCorners = false

function DS_AddAnimPoint:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu(MOHO.Localize("/Scripts/Tool/AddPoint/SelectGroup=Select Group"))

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	--layout:AddChild(self.popup)

	self.autoWeldCheck = LM.GUI.CheckBox(MOHO.Localize("/Scripts/Tool/AddPoint/AutoWeld=Auto-weld"), self.AUTOWELD)
	--layout:AddChild(self.autoWeldCheck)

	self.autoFillCheck = LM.GUI.CheckBox(MOHO.Localize("/Scripts/Tool/AddPoint/AutoFill=Auto-fill"), self.AUTOFILL)
	--layout:AddChild(self.autoFillCheck)

	self.sharpCornersCheck = LM.GUI.CheckBox(MOHO.Localize("/Scripts/Tool/AddPoint/SharpCorners=Sharp corners"), self.SHARPCORNERS)
	layout:AddChild(self.sharpCornersCheck)
end

function DS_AddAnimPoint:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)

	self.autoWeldCheck:SetValue(LM_TranslatePoints.autoWeld)
	self.autoFillCheck:SetValue(LM_TranslatePoints.autoFill)
	self.sharpCornersCheck:SetValue(DS_AddAnimPoint.sharpCorners)
end

function DS_AddAnimPoint:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.AUTOWELD) then
		LM_TranslatePoints.autoWeld = self.autoWeldCheck:Value()
	elseif (msg == self.AUTOFILL) then
		LM_TranslatePoints.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.SHARPCORNERS) then
		DS_AddAnimPoint.sharpCorners = self.sharpCornersCheck:Value()
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
