-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_colortool"

-- **************************************************
-- General information about this script
-- **************************************************

DS_colortool = {}

DS_colortool.BASE_STR = 2320

function DS_colortool:Name()
	return "Color tool"
end

function DS_colortool:Version()
	return "1.1"
end

function DS_colortool:Description()
	return "Does a lot of stuff with color"
end


function DS_colortool:Creator()
	return "dauid.com"
end

function DS_colortool:UILabel()
	return "Color tool"
end

-- **************************************************
-- Recurring values
-- **************************************************

DS_colortool.prevMousePt = LM.Point:new_local()
DS_colortool.prevSelID = -1
DS_colortool.handleRadius = 0.03
DS_colortool.dragMode = -1 --0:shape picker, 1:color picker, 2:color pusher, 3:center point, 4:handle point
DS_colortool.selShape = nil
DS_colortool.startVec = LM.Vector2:new_local()
DS_colortool.lastVec = LM.Vector2:new_local()
DS_colortool.GlobalFillColor = nil
DS_colortool.GlobalStrokeColor = nil
DS_colortool.Fill1 = nil
DS_colortool.Stroke1 = nil
DS_colortool.Fill2 = nil
DS_colortool.Stroke2 = nil
DS_colortool.Fill3 = nil
DS_colortool.Stroke3 = nil
DS_colortool.Fill4 = nil
DS_colortool.Stroke4 = nil
DS_colortool.DoDocument = false
DS_colortool.ChangeFills = true
DS_colortool.ChangeStrokes = true
DS_colortool.Tolerance = 1
DS_colortool.darkLine = 0.16


-- **************************************************
-- The guts of this script
-- **************************************************

function DS_colortool:IsEnabled(moho)
		return true
end

function DS_colortool:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.prevMousePt:Set(mouseEvent.pt)
	self.prevSelID = mouseEvent.view:PickShape(mouseEvent.pt)
	self.selShape = nil

	self.dragMode = 0 -- shape picker (default)
	if (mouseEvent.altKey) then
		self.dragMode = 1 -- color picker
		if (mouseEvent.ctrlKey) then
			self.dragMode = 2 -- color pusher
		end
	else
		-- cycle through shapes and see if the user clicked on an effect handle
		local g = mouseEvent.view:Graphics()
		local matrix = LM.Matrix:new_local()
		moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
		g:Push()
		g:ApplyMatrix(matrix)
		local graphicsScale = g:CurrentScale(false)
		g:Pop()

		for i = 0, mesh:CountShapes() - 1 do
			local shape = mesh:Shape(i)
			if (shape:HasPositionDependentStyles()) then
				local center = shape:EffectHandle1()
				local handleLocation = shape:EffectHandle2()

				if ((center - mouseEvent.vec):Mag() < self.handleRadius / graphicsScale) then
					self.dragMode = 3
					self.selShape = shape
					self.startVec:Set(shape.fEffectOffset.value)
					break
				elseif ((handleLocation - mouseEvent.vec):Mag() < self.handleRadius / graphicsScale) then
					self.dragMode = 4
					self.selShape = shape
					self.startVec:Set(handleLocation)
					self.lastVec:Set(mouseEvent.vec)
					break
				end
			end
		end
	end

	if (self.dragMode == 1 or self.dragMode == 2) then -- color/style picker/pusher
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		if (self.prevSelID >= 0) then
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected and i ~= self.prevSelID) then
					local sourceShape = mesh:Shape(self.prevSelID)
					local destShape = mesh:Shape(i)
					if (self.dragMode == 2) then -- push
						sourceShape:CopyStyleProperties(destShape)
					else -- pick
						destShape:CopyStyleProperties(sourceShape)
					end
					break
				end
			end
		end
	elseif (self.dragMode == 0) then -- shape picker
		for i = 0, mesh:CountShapes() - 1 do
			mesh:Shape(i).fSelected = false
		end
		if (self.prevSelID >= 0) then
			mesh:Shape(self.prevSelID).fSelected = true
		end
		moho:UpdateSelectedChannels()
	elseif (self.dragMode == 3 or self.dragMode == 4) then -- move an effect handle
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
	end

	mouseEvent.view:DrawMe()
end

function DS_colortool:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.prevMousePt:Set(mouseEvent.pt)
	self.prevSelID = mouseEvent.view:PickShape(mouseEvent.pt)

	if (self.dragMode == 1 or self.dragMode == 2) then -- color/style picker/pusher
		if (self.prevSelID >= 0) then
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected and i ~= self.prevSelID) then
					local sourceShape = mesh:Shape(self.prevSelID)
					local destShape = mesh:Shape(i)
					if (self.dragMode == 2) then -- push
						sourceShape:CopyStyleProperties(destShape)
					else -- pick
						destShape:CopyStyleProperties(sourceShape)
					end
					break
				end
			end
		end
	elseif (self.dragMode == 0) then -- shape picker
		for i = 0, mesh:CountShapes() - 1 do
			mesh:Shape(i).fSelected = false
		end
		if (self.prevSelID >= 0) then
			mesh:Shape(self.prevSelID).fSelected = true
		end
		moho:UpdateSelectedChannels()
	elseif (self.dragMode == 3 or self.dragMode == 4) then -- move an effect handle
		if (self.selShape ~= nil) then
			if (self.dragMode == 3) then
				self.selShape.fEffectOffset:SetValue(moho.layerFrame, self.startVec + (mouseEvent.vec - mouseEvent.startVec))
			else
				local center = self.selShape:EffectHandle1()
				local handleLocation = self.selShape:EffectHandle2()
				local min = LM.Vector2:new_local()
				local max = LM.Vector2:new_local()
				self.selShape:ShapeBounds(min, max, 0)
				handleLocation = self.startVec + (mouseEvent.vec - mouseEvent.startVec)
				local v = handleLocation - center
				local v1 = self.lastVec - center
				local v2 = mouseEvent.vec - center
				v2:Rotate(-math.atan2(v1.y, v1.x))
				local angle = math.atan2(v2.y, v2.x)
				angle = angle + self.selShape.fEffectRotation.value
				self.lastVec:Set(mouseEvent.vec)
				self.selShape.fEffectRotation:SetValue(moho.layerFrame, angle)
				if (mouseEvent.shiftKey) then
					self.selShape.fEffectRotation:SetValue(moho.layerFrame, self.selShape.fEffectRotation.value / (math.pi / 4))
					self.selShape.fEffectRotation:SetValue(moho.layerFrame, (math.pi / 4) * LM.Round(self.selShape.fEffectRotation.value))
				end
				if (max.y - min.y > max.x - min.x) then
					self.selShape.fEffectScale:SetValue(moho.layerFrame, v:Mag() / ((max.y - min.y) / 2.0))
				else
					self.selShape.fEffectScale:SetValue(moho.layerFrame, v:Mag() / ((max.x - min.x) / 2.0))
				end
			end
			moho:NewKeyframe(CHANNEL_FXXFORM)
			moho:UpdateSelectedChannels()
		end
	end

	mouseEvent.view:DrawMe()
end

function DS_colortool:OnKeyDown(moho, keyEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (keyEvent.keyCode == LM.GUI.KEY_UP) then
		if (keyEvent.ctrlKey) then -- select higher in the stacking order
			self.prevSelID = self.prevSelID + 1
			local id = keyEvent.view:PickShape(self.prevMousePt, self.prevSelID)
			for i = 0, mesh:CountShapes() - 1 do
				mesh:Shape(i).fSelected = false
			end
			if (id >= 0) then
				mesh:Shape(id).fSelected = true
			end
			moho:UpdateSelectedChannels()
		elseif (keyEvent.shiftKey) then -- raise shape to top
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:RaiseShape(i, true)
					break
				end
			end
		else -- raise shape
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:RaiseShape(i, false)
					break
				end
			end
		end
		keyEvent.view:DrawMe()
	elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
		if (keyEvent.ctrlKey) then -- select lower in the stacking order
			self.prevSelID = self.prevSelID - 1
			id = keyEvent.view:PickShape(self.prevMousePt, self.prevSelID)
			for i = 0, mesh:CountShapes() - 1 do
				mesh:Shape(i).fSelected = false
			end
			if (id >= 0) then
				mesh:Shape(id).fSelected = true
			end
			moho:UpdateSelectedChannels()
		elseif (keyEvent.shiftKey) then -- lower shape to bottom
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:LowerShape(i, true)
					break
				end
			end
		else -- lower shape
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:LowerShape(i, false)
					break
				end
			end
		end
		keyEvent.view:DrawMe()
	elseif ((keyEvent.keyCode == LM.GUI.KEY_DELETE) or (keyEvent.keyCode == LM.GUI.KEY_BACKSPACE)) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		local i = 0
		while i < mesh:CountShapes() do
			if (mesh:Shape(i).fSelected) then
				mesh:DeleteShape(i)
			else
				i = i + 1
			end
		end
		keyEvent.view:DrawMe()
	end
end

function DS_colortool:DrawMe(moho, view)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local g = view:Graphics()
	local matrix = LM.Matrix:new_local()

	g:SetSmoothing(true)
	g:SetBezierTolerance(2.0)

	moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
	g:Push()
	g:ApplyMatrix(matrix)

	local graphicsScale = g:CurrentScale(false)

	for i = 0, mesh:CountShapes() - 1 do
		local shape = mesh:Shape(i)
		if (shape:HasPositionDependentStyles()) then
			local center = shape:EffectHandle1()
			local handleLocation = shape:EffectHandle2()

			g:SetColor(255, 0, 0, 96)
			g:FillCircle(center, self.handleRadius / graphicsScale)
			g:SetColor(255, 0, 0)
			g:FrameCircle(center, self.handleRadius / graphicsScale)
			g:FrameCircle(handleLocation, self.handleRadius / graphicsScale)

			local v = handleLocation - center
			v:NormMe()
			v = v * self.handleRadius / graphicsScale
			center = center + v
			handleLocation = handleLocation - v
			g:DrawLine(center.x, center.y, handleLocation.x, handleLocation.y)
		end
	end

	g:Pop()
	g:SetSmoothing(false)
end
function DS_colortool:Swap(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	local FillColor = LM.ColorVector:new_local() 
	local strokeColor = LM.ColorVector:new_local() 
	
	for i=0, mesh:CountShapes()-1 do
	local theShape=mesh:Shape(i)
		if (theShape.fSelected) then
		FillColor=theShape.fMyStyle.fFillCol:GetValue(moho.frame)
		StrokeColor=theShape.fMyStyle.fLineCol:GetValue(moho.frame)
		theShape.fMyStyle.fFillCol:SetValue(moho.frame, StrokeColor)
		theShape.fMyStyle.fLineCol:SetValue(moho.frame, FillColor)
		
		end
	end

end
	
function DS_colortool:SetStroke(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	local FillColor = LM.ColorVector:new_local() 
	local StrokeColor = LM.ColorVector:new_local() 
	
	for i=0, mesh:CountShapes()-1 do
	local theShape=mesh:Shape(i)
		if (theShape.fSelected) then
		FillColor=theShape.fMyStyle.fFillCol:GetValue(moho.frame)
		StrokeColor=theShape.fMyStyle.fLineCol:GetValue(moho.frame)
		theShape.fMyStyle.fLineCol:SetValue(moho.frame, FillColor)
		
		end
	end
	
end

function DS_colortool:ColorStroke(moho, darker)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	local FillColor = LM.ColorVector:new_local() 
	local StrokeColor = LM.ColorVector:new_local() 
	
	for i=0, mesh:CountShapes()-1 do
	local theShape=mesh:Shape(i)
		if (theShape.fSelected) then
		--FillColor=theShape.fMyStyle.fFillCol:GetValue(moho.frame)
		StrokeColor=theShape.fMyStyle.fFillCol:GetValue(moho.frame)
		--local darker = 0.16
		local nr = StrokeColor.r - darker
		local ng = StrokeColor.g - darker
		local nb = StrokeColor.b - darker
		local na = StrokeColor.a
		StrokeColor:Set(nr, ng, nb, na)
		theShape.fMyStyle.fLineCol:SetValue(moho.frame, StrokeColor)
		
		end
	end
	
end

function DS_colortool:SetFill(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	local FillColor = LM.ColorVector:new_local() 
	local StrokeColor = LM.ColorVector:new_local() 
	
	for i=0, mesh:CountShapes()-1 do
	local theShape=mesh:Shape(i)
		if (theShape.fSelected) then
		FillColor=theShape.fMyStyle.fFillCol:GetValue(moho.frame)
		StrokeColor=theShape.fMyStyle.fLineCol:GetValue(moho.frame)
		theShape.fMyStyle.fFillCol:SetValue(moho.frame, StrokeColor)
		
		end
	end
end
	
function DS_colortool:GetColor(moho)
	mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	
	for i=0, mesh:CountShapes()-1 do
	theShape=mesh:Shape(i)
		if (theShape.fSelected) then
		self.GlobalFillColor=theShape.fMyStyle.fFillCol:GetValue(moho.frame):AsColorStruct()
		self.GlobalStrokeColor=theShape.fMyStyle.fLineCol:GetValue(moho.frame):AsColorStruct()

		end
	end
end

function DS_colortool:CheckIfGroupLayer(moho) -- This is just a function I wrote in order to avoid writing the line below several times.
	if ((moho.layer:LayerType() == MOHO.LT_GROUP) or (moho.layer:LayerType() == MOHO.LT_SWITCH) or (moho.layer:LayerType() == MOHO.LT_BONE)) then
		return true
	end
end


function DS_colortool:GoThroughLayers(moho, oldFill, oldStroke, newFill, newStroke )
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	local mesh = moho:Mesh()
	
	if (self.DoDocument) then -- DoDocument means change colors in every layer
		local numlayers = moho.document:CountLayers() --Do top layers
		for i = 0, numlayers - 1 do
			moho:SetSelLayer(moho.document:Layer(i)) 
			----------Do group layers
			if (DS_colortool:CheckIfGroupLayer(moho)) then
				layer = moho:LayerAsGroup(moho.document:Layer(i))
				for s= 0, layer:CountLayers()-1 do
				moho:SetSelLayer(layer:Layer(s)) 
				--- Do group layers within group layers
					if (DS_colortool:CheckIfGroupLayer(moho)) then
						slayer = moho:LayerAsGroup(layer:Layer(s))
						for ss= 0, slayer:CountLayers()-1 do
							moho:SetSelLayer(slayer:Layer(ss))
							------ Do group layers within group layers within group layers... Phew!
							if (DS_colortool:CheckIfGroupLayer(moho)) then
							sublayer = moho:LayerAsGroup(slayer:Layer(ss))
								for sub= 0, sublayer:CountLayers()-1 do
								moho:SetSelLayer(sublayer:Layer(sub))
								------ Do group layers within group layers within group layers withing group layers... Argh! Thats it!
									if (DS_colortool:CheckIfGroupLayer(moho)) then
									subsublayer = moho:LayerAsGroup(sublayer:Layer(sub))
										for subsub= 0, subsublayer:CountLayers()-1 do
										moho:SetSelLayer(subsublayer:Layer(subsub))
										DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
										end
									else
									DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
									end
								end
							else
							DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
							end
						end
					else
					DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
					end
				end
			else
			DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
			end
		end
	else -- The following is if we only want to change colors in the current layer.
		if (DS_colortool:CheckIfGroupLayer(moho)) then
				layer = moho:LayerAsGroup(moho.layer)
				for s= 0, layer:CountLayers()-1 do
				moho:SetSelLayer(layer:Layer(s)) 
				--- Do group layers within group layers
					if (DS_colortool:CheckIfGroupLayer(moho)) then
						slayer = moho:LayerAsGroup(layer:Layer(s))
						for ss= 0, slayer:CountLayers()-1 do
							moho:SetSelLayer(slayer:Layer(ss))
							------ Do group layers within group layers within group layers... Phew!
							if (DS_colortool:CheckIfGroupLayer(moho)) then
							sublayer = moho:LayerAsGroup(slayer:Layer(ss))
								for sub= 0, sublayer:CountLayers()-1 do
								moho:SetSelLayer(sublayer:Layer(sub))
								------ Do group layers within group layers within group layers withing group layers... Argh! Thats it!
									if (DS_colortool:CheckIfGroupLayer(moho)) then
									subsublayer = moho:LayerAsGroup(sublayer:Layer(sub))
										for subsub= 0, subsublayer:CountLayers()-1 do
										moho:SetSelLayer(subsublayer:Layer(subsub))
										DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
										end
									else
									DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
									end
								end
							else
							DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
							end
						end
					else
					DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
					end
				end
			else
			DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
			end
	end
	
end

-- **************************************************
-- Replace color function, based on the script js_replace_color by Julien Stiegler
-- **************************************************

function DS_colortool:replaceColor(moho, oldFill, oldStroke, newFill, newStroke )
	if (moho.layer:LayerType() == MOHO.LT_VECTOR) then 
			local mesh = moho:Mesh()
				for j = mesh:CountShapes() -1, 0, -1 do
					local shape = mesh:Shape(j)
					local fColor = shape.fMyStyle.fFillCol:GetValue(0):AsColorStruct()
					
					if(self.ChangeFills and (DS_colortool:checkTolerance(oldFill.r, fColor.r)) and (DS_colortool:checkTolerance(oldFill.g, fColor.g)) and (DS_colortool:checkTolerance(oldFill.b, fColor.b)) and (DS_colortool:checkTolerance(oldFill.a, fColor.a))) then
						shape.fMyStyle.fFillCol:SetValue(0, newFill)
					end
					local lColor = shape.fMyStyle.fLineCol:GetValue(0):AsColorStruct()
					if(self.ChangeStrokes and (DS_colortool:checkTolerance(oldStroke.r, lColor.r)) and (DS_colortool:checkTolerance(oldStroke.g, lColor.g)) and (DS_colortool:checkTolerance(oldStroke.b, lColor.b)) and (DS_colortool:checkTolerance(oldStroke.a, lColor.a))) then
						shape.fMyStyle.fLineCol:SetValue(0, newStroke)
					end
				end 
			end
end

function DS_colortool:checkTolerance(oldColor, newColor)
	local theTolerance=self.Tolerance
	local low = oldColor - theTolerance
	local high = oldColor + theTolerance
	if ((newColor > low) and (newColor < high)) then
	return true
	end
end

	
	
-- **************************************************
-- Menu
-- **************************************************

DS_colortool.SWAP = MOHO.MSG_BASE
DS_colortool.STROKE = MOHO.MSG_BASE +1
DS_colortool.FILL = MOHO.MSG_BASE +2
DS_colortool.COPY1 = MOHO.MSG_BASE +3
DS_colortool.PASTE1 = MOHO.MSG_BASE +4
DS_colortool.COPY2 = MOHO.MSG_BASE +5
DS_colortool.PASTE2 = MOHO.MSG_BASE +6
DS_colortool.INFO = MOHO.MSG_BASE +7
DS_colortool.CHANGEFILLS = MOHO.MSG_BASE +8
DS_colortool.CHANGESTROKES = MOHO.MSG_BASE +9
DS_colortool.DOCUMENT = MOHO.MSG_BASE +10
DS_colortool.REPLACE = MOHO.MSG_BASE +11
DS_colortool.TOLERANCE = MOHO.MSG_BASE +12
DS_colortool.COLSTROKE = MOHO.MSG_BASE +13
DS_colortool.DARKM = MOHO.MSG_BASE +14

function DS_colortool:DoLayout(moho, layout)



	self.swapBut= LM.GUI.Button("Swap", self.SWAP)
	layout:AddChild(self.swapBut)
	
	self.strokeBut= LM.GUI.Button("Stroke to fill", self.STROKE)
	layout:AddChild(self.strokeBut)
	
	self.fillBut= LM.GUI.Button("Fill to stroke", self.FILL)
	layout:AddChild(self.fillBut)
	
	self.cstrokeBut= LM.GUI.Button("Color stroke", self.COLSTROKE)
	layout:AddChild(self.cstrokeBut)
	
	layout:AddChild(LM.GUI.StaticText("Dark:"))
	self.dark = LM.GUI.TextControl(0, "0.000", self.DARKM, LM.GUI.FIELD_FLOAT)
	self.dark:SetWheelInc(0.1)
	self.dark:SetValue(self.darkLine)
	layout:AddChild(self.dark)
	
	layout:AddChild(LM.GUI.Divider(true))
	
	self.fill1swatch = LM.GUI.ColorSwatch(true)
	if (self.Fill1~=nil) then
		self.fill1swatch:SetValue(self.Fill1)
	end
	layout:AddChild( self.fill1swatch)
	
	self.stroke1swatch = LM.GUI.ColorSwatch(true)
	if (self.Stroke1~=nil) then
		self.stroke1swatch:SetValue(self.Stroke1)
	end
	layout:AddChild( self.stroke1swatch)
	
	self.copy1= LM.GUI.Button("Copy", self.COPY1)
	layout:AddChild(self.copy1)
	
	self.paste1= LM.GUI.Button("Paste", self.PASTE1)
	layout:AddChild(self.paste1)
	
	--***********************************************
	
	layout:AddChild(LM.GUI.Divider(true))
	
	self.fill2swatch = LM.GUI.ColorSwatch(true)
	if (self.Fill2~=nil) then
		self.fill2swatch:SetValue(self.Fill2)
	end
	layout:AddChild( self.fill2swatch)
	
	self.stroke2swatch = LM.GUI.ColorSwatch(true)
	if (self.Stroke2~=nil) then
		self.stroke2swatch:SetValue(self.Stroke2)
	end
	layout:AddChild( self.stroke2swatch)
	
	self.copy2= LM.GUI.Button("Copy", self.COPY2)
	layout:AddChild(self.copy2)
	
	self.paste2= LM.GUI.Button("Paste", self.PASTE2)
	layout:AddChild(self.paste2)
	
	--***********************************************
	
	
	
	layout:AddChild(LM.GUI.Divider(true))
	self.infobutton= LM.GUI.Button("Info", self.INFO)
	layout:AddChild(self.infobutton)
	
	self.replacebutton= LM.GUI.Button("Replace", self.REPLACE)
	layout:AddChild(self.replacebutton)
	
	self.docubutt = LM.GUI.CheckBox("All layers", self.DOCUMENT)
	layout:AddChild(self.docubutt)
	
	self.fillbutt = LM.GUI.CheckBox("Fill", self.CHANGEFILLS)
	self.fillbutt:SetValue(true)
	layout:AddChild(self.fillbutt)
	
	self.strokebutt = LM.GUI.CheckBox("Stroke", self.CHANGESTROKES)
	self.strokebutt:SetValue(true)
	layout:AddChild(self.strokebutt)
	
	layout:AddChild(LM.GUI.StaticText("Tolerance:"))
	self.tolerance = LM.GUI.TextControl(0, "000", self.TOLERANCE, LM.GUI.FIELD_UINT)
	self.tolerance:SetWheelInc(1)
	self.tolerance:SetValue(1)
	layout:AddChild(self.tolerance)

end


function DS_colortool:HandleMessage(moho, view, msg)

	if (msg == self.SWAP) then
		DS_colortool:Swap(moho)
	end
	if (msg == self.COLSTROKE) then
		DS_colortool:ColorStroke(moho,self.dark:Value())
	end
	if (msg == self.STROKE) then
		DS_colortool:SetStroke(moho)
	end
	if (msg == self.FILL) then
		DS_colortool:SetFill(moho)
	end
	if (msg == self.COPY1) then
		DS_colortool:GetColor(moho)
		self.fill1swatch:SetValue(self.GlobalFillColor)
		self.stroke1swatch:SetValue(self.GlobalStrokeColor)
		self.Fill1 = self.GlobalFillColor
		self.Stroke1 = self.GlobalStrokeColor
	end
	if (msg == self.PASTE1) then
		local mesh = moho:Mesh()
		if (mesh == nil) then
			return
		end

		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		for i=0, mesh:CountShapes()-1 do
		local theShape=mesh:Shape(i)
			if (theShape.fSelected) then
			theShape.fMyStyle.fFillCol:SetValue(moho.frame, self.fill1swatch:Value())
			theShape.fMyStyle.fLineCol:SetValue(moho.frame, self.stroke1swatch:Value())
			
			end
		end
	end
	
	if (msg == self.COPY2) then
		DS_colortool:GetColor(moho)
		self.fill2swatch:SetValue(self.GlobalFillColor)
		self.stroke2swatch:SetValue(self.GlobalStrokeColor)
		self.Fill2 = self.GlobalFillColor
		self.Stroke2 = self.GlobalStrokeColor
	end
	if (msg == self.PASTE2) then
		local mesh = moho:Mesh()
		if (mesh == nil) then
			return
		end

		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		for i=0, mesh:CountShapes()-1 do
		local theShape=mesh:Shape(i)
			if (theShape.fSelected) then
			theShape.fMyStyle.fFillCol:SetValue(moho.frame, self.fill2swatch:Value())
			theShape.fMyStyle.fLineCol:SetValue(moho.frame, self.stroke2swatch:Value())
			
			end
		end
	end

	if (msg == self.INFO) then
		if (self.GlobalFillColor ~=nil) then
			print("Last copied fill:")
			print("Red: " .. self.GlobalFillColor.r)
			print("Blue: " .. self.GlobalFillColor.b)
			print("Green: " .. self.GlobalFillColor.g)
			print("Opacity: " .. self.GlobalFillColor.a)
			--print("Hex: #" .. DS_colortool:Hex(self.GlobalFillColor.r) .. DS_colortool:Hex(self.GlobalFillColor.g) .. DS_colortool:Hex(self.GlobalFillColor.b))
		else
			print("No color has been copied yet.")
		end
	end
	
	if (msg == self.REPLACE) then
	self.Tolerance=self.tolerance:IntValue()
		DS_colortool:GoThroughLayers(moho, self.fill1swatch:Value() , self.stroke1swatch:Value(), self.fill2swatch:Value() , self.stroke2swatch:Value())
	end
	if (msg == self.DOCUMENT) then
		self.DoDocument=self.docubutt:Value()
	end
	if (msg == self.CHANGEFILLS) then
		self.ChangeFills=self.fillbutt:Value()
	end
	if (msg == self.CHANGESTROKES) then
		self.ChangeStrokes=self.strokebutt:Value()
	end
	if (msg == self.TOLERANCE) then
		if (self.Tolerance>256) then
		self.tolerance:SetValue(256)
		end
		if (self.Tolerance<1) then
		self.tolerance:SetValue(1)
		end
		self.Tolerance=self.tolerance:IntValue()
	end
	if (msg == self.DARKM) then
	self.darkLine = self.dark:Value()
	end
	moho:UpdateUI()
end
