
-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_layertosvg"

-- **************************************************
-- General information about this script
-- **************************************************

DS_layertosvg = {}
function DS_layertosvg:Name()
	return "Layer to SVG"
end

function DS_layertosvg:Version()
	return "0.1"
end

function DS_layertosvg:Description()
	return "Export layer as SVG"
end

function DS_layertosvg:Creator()
	return "dauid.com"
end

function DS_layertosvg:UILabel()
	return("Layer to SVG")
end


-- **************************************************
-- The guts of this script
-- **************************************************


function DS_layertosvg:Run(moho)

	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

local path = LM.GUI.SaveFile("Select Data File")
	if (path == "") then
		return
	end
	
		local f = io.open(path..".svg", "w")

	if (f == nil) then
		return
	end

local width = moho.document:Width()
local height = moho.document:Height()
local aspect = moho.document:AspectRatio()
local newX = (moho.document:Width()/2)
local newY = moho.document:Height()/2
local newZ = moho.document:Height()/2
local startframe = moho.document:StartFrame()
local endframe = moho.document:EndFrame()
local anchor= LM.Vector2:new_local()
local translation= LM.Vector3:new_local()
local layer =moho.layer
local scale= LM.Vector3:new_local()

if (layer:LayerType()==MOHO.LT_IMAGE) then

layer = moho:LayerAsImage(layer)
 pixelX=layer:PixelWidth()
 pixelY=layer:PixelHeight()
end

f:write("<?xml version='1.0' standalone='no'?>\n")
f:write(" <!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.1//EN' 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'>\n")
f:write(" <svg xmlns='http://www.w3.org/2000/svg' version='1.1'  width='2"..width.."px' height='"..height.."px' viewBox='0 0 "..width.." "..height.."' zoomAndPan='disable'>\n")
f:write("<?xml version='1.0' standalone='no'?>\n")
	
	local curveID = 0
    local segID = 0

for s = 0, mesh:CountShapes() - 1 do
	local shape = mesh:Shape(s)
	for e = 0, shape:CountEdges()-1 do
		curveID, segID = shape:GetEdge(i, curveID, segID)
		curve = mesh:Curve(curveID)
		for c = 0, curve:CountPoints() do
			
		end
	end
			
end

	anchor = layer:Origin()
	
if (layer:LayerType()==MOHO.LT_IMAGE) then
	if (aspect >0) then
		anchor.x = ((anchor.x + 1) * newY)+(newX-newY) -(newX-(pixelX/2))
		anchor.y = (((anchor.y*(-1)) + 1) * newY)-(newY-(pixelY/2))
	else
		anchor.x = ((anchor.x + 1) * newX) -(newX-(pixelX/2))
		anchor.y = ((anchor.y*(-1)) + 1) * newX + (newY-newX)-(newY-(pixelY/2))
	end
	f:write("\t".. startframe .."\t".. anchor.x .."\t".. anchor.y .."\t".. "0" .."\n")
else
	if (aspect >0) then
		anchor.x = ((anchor.x + 1) * newY)+(newX-newY)
		anchor.y = ((anchor.y*(-1)) + 1) * newY
	else
		anchor.x = ((anchor.x + 1) * newX)
		anchor.y = ((anchor.y*(-1)) + 1) * newX + (newY-newX)
	end
	f:write("\t".. startframe .."\t".. anchor.x.."\t".. anchor.y .."\t".. "0" .."\n")
end

--Position
f:write("\nTransform\tPosition\n")
f:write("\tFrame\tX pixels\tY pixels\tZ pixels\n")
	anchor = layer:Origin()
	for i = startframe, endframe do
	translation = layer.fTranslation:GetValue(i)
	if (aspect >0) then
		translation.x = (((translation.x+anchor.x) + 1) * newY)+(newX-newY)
		translation.y = (((translation.y+anchor.y)*(-1)) + 1) * newY
		translation.z = ((translation.z*(-1))*newZ)
	else
		translation.x = (((translation.x+anchor.x) + 1) * newX)
		translation.y = (((translation.y+anchor.y)*(-1)) + 1) * newX + (newY-newX)
		translation.z = ((translation.z*(-1))*newX)
	end
	f:write("\t".. i .."\t".. translation.x.."\t".. translation.y .."\t".. translation.z .."\n")
	end 
	--scale
f:write("\nTransform\tScale\n")
f:write("\tFrame\tX percent\tY percent\tZ percent\n")
	for i = startframe, endframe do
	scale = layer.fScale:GetValue(i)
	f:write("\t".. i .."\t".. scale.x*100 .."\t".. scale.y*100 .."\t".. scale.z*100 .."\n")
	end 
--Rotation
f:write("\nTransform\tRotation\n")
f:write("\tFrame\tdegrees\n")
	for i = startframe, endframe do
	local rotation = math.deg(layer.fRotationZ:GetValue(i))
	f:write("\t".. i .."\t".. rotation*(-1) .."\n")
	end 
--X rotation
f:write("\nTransform\tX Rotation\n")
f:write("\tFrame\tdegrees\n")
	for i = startframe, endframe do
	local xrot = layer.fRotationX:GetValue(i)
	f:write("\t".. i .."\t".. math.deg(xrot) .."\n")
	end 
--Y rotation
f:write("\nTransform\tY Rotation\n")
f:write("\tFrame\tdegrees\n")
	for i = startframe, endframe do
	local yrot = layer.fRotationY:GetValue(i)
	f:write("\t".. i .."\t".. (math.deg(yrot))*(-1) .."\n")
	end 
--Opacity
f:write("\nTransform\tOpacity\n")
f:write("\tFrame\tpercent\n")
	for i = startframe, endframe do
		if (layer.fVisibility:GetValue(i)==false) then
		f:write("\t".. i .."\t".. 0 .."\n")
		else
		local opacity = layer.fAlpha:GetValue(i)
		f:write("\t".. i .."\t".. opacity*100 .."\n")
		end
	end 
f:write("</svg>")
f:close() --Close the file
end