--[[
Goal :
- import all valid files from a directory
Done:
-create a list of valid files using os commands (ls | dir)
-open the list
-import all the valid files
Todo:
-check if linux/mac version works
-let user enter a file name filter (ex: "*.ai" "scene1*.*") passed to DIR/ls command
]]
-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_shade"

-- **************************************************
-- General information about this script
-- **************************************************

DS_shade = {}
function DS_shade:Name()
	return "DS Shade"
end

function DS_shade:Version()
	return "1.0"
end

function DS_shade:Description()
	return "Import bitmap or vector Import directory in a switch layer (files must have valid extension)"
end

function DS_shade:Creator()
	return "dauid.com"
end

function DS_shade:UILabel()
	return("DS shade")
end


-- **************************************************
-- The guts of this script
-- **************************************************


function DS_shade:OnMouseDown(moho, mouseEvent)
	self:Run(moho)
end

function DS_shade:Run(moho)


local originalLayer = moho.layer --The layer selected at start
local originalName = originalLayer:Name()

local theGroupLayer = (moho:CreateNewLayer(MOHO.LT_GROUP)) -- The group layer that contains everything
theGroupLayer:SetName(originalName .. " with shade")
layer = moho:LayerAsGroup(theGroupLayer)
layer:SetGroupMask(MOHO.GROUP_MASK_HIDE_ALL)

local theSecondGroupLayer = (moho:CreateNewLayer(MOHO.LT_GROUP)) -- The group layer that contains the shade
moho:PlaceLayerInGroup(originalLayer, theGroupLayer)
theSecondGroupLayer:SetName(originalName .. " shade")


layer = moho:LayerAsGroup(theSecondGroupLayer)
layer:SetGroupMask(MOHO.GROUP_MASK_HIDE_ALL)
moho.layer.fAlpha:SetValue(0, 0.14)
moho.layer.fBlur:SetValue(0, 0.01)


local theCopyLayer = moho:DuplicateLayer(originalLayer) -- Copy the original layer and place it in the shade group layer
theCopyLayer:SetName("Copy of " .. originalName)
moho:PlaceLayerInGroup(theCopyLayer, theSecondGroupLayer, false)
moho:PlaceLayerInGroup(theSecondGroupLayer, theGroupLayer)

--theCopyLayer = moho:LayerAsBone(theCopyLayer)

--local moveLayerVal = LM.Vector3:new_local()
--moveLayerVal.x = -0.5
--moveLayerVal.y = -0.5
--moveLayerVal.z = 0
--theCopyLayer.fTranslation:SetValue(0, moveLayerVal)

local blackLayer = (moho:CreateNewLayer(MOHO.LT_VECTOR)) -- Create a layer with black
blackLayer:SetName("Black")
moho:PlaceLayerInGroup(blackLayer, theSecondGroupLayer)

local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
theAspect=moho.document:AspectRatio()
	local FillColor = LM.ColorVector:new_local() 
	FillColor:Set(0.0, 0.0, 0.0) 
	
	self:BuildPolygon(moho, mesh)
	local id=moho:CreateShape(true)
	mesh:Shape(id).fMyStyle.fFillCol:SetValue(0, FillColor:AsColorStruct())
	mesh:Shape(id).fHasOutline = false
	
	moho:SetSelLayer(theCopyLayer)


end

function DS_shade:BuildPolygon(moho, mesh)

	local v = LM.Vector2:new_local()
	mesh:SelectNone()
	
	v.x=-1*theAspect
	v.y=1
	
local pointOne=(mesh:AddLonePoint(v, 0))
	v.x=1*theAspect
	v.y=1
local pointTwo=(mesh:AppendPoint(v, 0))

	v.x=1*theAspect
	v.y=-1
local pointThree=(mesh:AppendPoint(v, 0))

	v.x=-1*theAspect
	v.y=-1
local pointFour=(mesh:AppendPoint(v, 0))

	v.x=-1*theAspect
	v.y=1
local pointFive=(mesh:AppendPoint(v, 0))
	
mesh:WeldPoints(0, 4, 0)

	for i = 0, 3 do
		mesh:Point(i):SetCurvature(0.391379, moho.layerFrame)
	end

	mesh:SelectConnected()
end