-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_zposandscale"

-- **************************************************
-- General information about this script
-- **************************************************

DS_zposandscale = {}


function DS_zposandscale:Name()
	return "Translate Layer Z and Scale"
end

function DS_zposandscale:Version()
	return "0.5"
end

function DS_zposandscale:Description()
	return "Move layer back in Z space and scale relatively to maintain scale in view"
end

function DS_zposandscale:Creator()
	return "Smith Micro Software, Inc."
end

function DS_zposandscale:UILabel()
	return"Translate Layer Z and Scale"
end


-- **************************************************
-- Recurring values
-- **************************************************


-- **************************************************
-- The guts of this script
-- **************************************************
function DS_zposandscale:IsEnabled(moho)

	return true
end

function DS_zposandscale:OnMouseDown(moho, mouseEvent)

	moho.document:PrepMultiUndo()
	moho.document:SetDirty()
	curFrame=moho.layer:CurFrame()
	previousValue=0
	theScale=0
	layer=moho.layer



end


function DS_zposandscale:OnMouseMoved(moho, mouseEvent)


	mouseGenValue=(mouseEvent.pt.y-mouseEvent.startPt.y)
	local SmallerValue, plusOrMinus
	
	if (mouseEvent.shiftKey) then
		SmallerValue=(mouseGenValue/1000)
	else
		SmallerValue=(mouseGenValue/100)
	end
	plusOrMinus=(SmallerValue-previousValue)
	-----------------------------------------------------------------
	local layer=moho.layer
	local newVal = LM.Vector3:new_local()
	local newScale = LM.Vector3:new_local()
	local scaleVal = LM.Vector3:new_local()
	local camTrack = LM.Vector3:new_local()
	local layerOrigin = LM.Vector2:new_local()
	local newOrigin = LM.Vector2:new_local()
	camTrack = moho.document.fCameraTrack:GetValue(moho.frame)
	local theRatio = 1/camTrack.z
	
	moho.document:PrepMultiUndo()
	moho.document:SetDirty()
		
	transVal=layer.fTranslation:GetValue(moho.frame) 	-- Get the current position of the layer
	layerOrigin = layer:Origin() 						-- Get origin of layer
	newOrigin.x = transVal.x*(-1)						-- Get new origin by compensating for layer translation
	newOrigin.y = transVal.y*(-1)						
		layer:SetOrigin(newOrigin)
		
		theTrans=(transVal.z + plusOrMinus) 					-- Get the number from the Z input box
		
		scaleVal=layer.fScale:GetValue(moho.frame) 			-- Get the current scale of the layer
		scaleVal.x=(scaleVal.x*(1+((theTrans*(-1))*(1/camTrack.z))))*(1/scaleVal.x) --Goddamn formulas...
		scaleVal.y=(scaleVal.y*(1+((theTrans*(-1))*(1/camTrack.z))))*(1/scaleVal.y)
		scaleVal.z=(scaleVal.z*(1+((theTrans*(-1))*(1/camTrack.z))))*(1/scaleVal.z)
		
		layer.fScale:SetValue(moho.frame, scaleVal)
		
		transVal.z=theTrans					-- Change only Z in the vector that holds the layer translation
		layer.fTranslation:SetValue(moho.frame, transVal) 	-- Move the layer according to transVal vector values
		
		
		moho:NewKeyframe(CHANNEL_LAYER_T)					-- And finally, add keyframes in the proper animation channels. S = Scale, T = Translation.
		moho:NewKeyframe(CHANNEL_LAYER_S)
	

	----------------------------------------------------------------------
	if (SmallerValue~=previousValue) then
	previousValue=SmallerValue
	end
	self.textZ:SetValue(moho.layer.fTranslation.value.z)
		mouseEvent.view:DrawMe()
		moho:UpdateSelectedChannels()

end





function DS_zposandscale:OnMouseUp(moho, mouseEvent, view)
moho:NewKeyframe(CHANNEL_LAYER_T)
moho:NewKeyframe(CHANNEL_LAYER_S)
moho:UpdateSelectedChannels()
mouseEvent.view:DrawMe()
	
		
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

DS_zposandscale.CHANGE = MOHO.MSG_BASE
DS_zposandscale.RESET = MOHO.MSG_BASE + 1
DS_zposandscale.IMAGEHEIGHT = MOHO.MSG_BASE + 2
DS_zposandscale.IMAGEWIDTH = MOHO.MSG_BASE + 3

function DS_zposandscale:DoLayout(moho, layout)

	layout:AddChild(LM.GUI.StaticText("Z:"))
	self.textZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textZ)

	self.resetBut= LM.GUI.Button("Reset", self.RESET)
	layout:AddChild(self.resetBut)
	self.imageheightBut= LM.GUI.Button("Scale image layer to match output height", self.IMAGEHEIGHT)
	layout:AddChild(self.imageheightBut)
	self.imagewidthBut= LM.GUI.Button("Scale image layer to match output width", self.IMAGEWIDTH)
	layout:AddChild(self.imagewidthBut)
end

function DS_zposandscale:UpdateWidgets(moho)
local theZ=moho.layer.fTranslation:GetValue(moho.frame)
self.textZ:SetValue(theZ.z)
end

function DS_zposandscale:HandleMessage(moho, view, msg)
	local layer=moho.layer
	local newVal = LM.Vector3:new_local()
	local newScale = LM.Vector3:new_local()
	local scaleVal = LM.Vector3:new_local()
	local camTrack = LM.Vector3:new_local()
	local layerOrigin = LM.Vector2:new_local()
	local newOrigin = LM.Vector2:new_local()
	camTrack = moho.document.fCameraTrack:GetValue(moho.frame)
	local theRatio = 1/camTrack.z

	if (msg == self.RESET) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()
				
				newVal.z=0
				newScale:Set(1,1,1)
				layer.fTranslation:SetValue(moho.frame, newVal)
				layer.fScale:SetValue(moho.frame, newScale)
		

		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_LAYER_T)
		moho:NewKeyframe(CHANNEL_LAYER_S)
	end
	if (msg == self.CHANGE) then
		moho.document:PrepMultiUndo()
		moho.document:SetDirty()
		
		
		transVal=layer.fTranslation:GetValue(moho.frame) 	-- Get the current position of the layer
		layerOrigin = layer:Origin() 						-- Get origin of layer
		newOrigin.x = transVal.x*(-1)						-- Get new origin by compensating for layer translation
		newOrigin.y = transVal.y*(-1)						
		layer:SetOrigin(newOrigin)
		
		theTrans=(self.textZ:FloatValue()) 					-- Get the number from the Z input box
		
		scaleVal=layer.fScale:GetValue(moho.frame) 			-- Get the current scale of the layer
		
		
		scaleVal.x=((scaleVal.x*(1+((theTrans*(-1))*(1/camTrack.z))))*(1/scaleVal.x)) --Goddamn formulas...
		scaleVal.y=((scaleVal.y*(1+((theTrans*(-1))*(1/camTrack.z))))*(1/scaleVal.y))
		scaleVal.z=((scaleVal.z*(1+((theTrans*(-1))*(1/camTrack.z))))*(1/scaleVal.z))
		
		
		
		layer.fScale:SetValue(moho.frame, scaleVal)
		
		transVal.z=self.textZ:FloatValue()					-- Change only Z in the vector that holds the layer translation
		layer.fTranslation:SetValue(moho.frame, transVal) 	-- Move the layer according to transVal vector values
		
		
		moho:NewKeyframe(CHANNEL_LAYER_T)					-- And finally, add keyframes in the proper animation channels. S = Scale, T = Translation.
		moho:NewKeyframe(CHANNEL_LAYER_S)
		
		end
	if (msg == self.IMAGEHEIGHT) then
		if (layer:LayerType()==MOHO.LT_IMAGE) then
		layer = moho:LayerAsImage(layer)
		scaleVal.y=moho.document:Height()/layer:PixelHeight()
		scaleVal.x=scaleVal.y
		layer.fScale:SetValue(moho.frame, scaleVal)
		end
	end
	if (msg == self.IMAGEWIDTH) then
		if (layer:LayerType()==MOHO.LT_IMAGE) then
		layer = moho:LayerAsImage(layer)
		scaleVal.x=moho.document:Width()/layer:PixelWidth()
		scaleVal.y=scaleVal.x
		layer.fScale:SetValue(moho.frame, scaleVal)
		end
	end
end
